
' Benutzung der AD-Ports auf der Open-Micro und Open-Mini
' von Dietmar Harlos am 6. Oktober 2005

' Auf dem PE-Board sitzt am Port PTA5 (PORT[6]) ein Poti, um eine veraender-
' bare Spannung zum Testen des Analog-Digital-Umsetzers (ADU) zu erzeugen.
' Im folgenden Beispiel wird gezeigt, was dabei zu beachten ist und es wird
' demonstriert, dass die OM in der Lage ist, den digitalisierten Wert auch
' als echten Spannungswert mit Nachkomma-Anteil auf der seriellen Schnitt-
' stelle auszugeben.


DEFINE adpoti AD[6]          'Poti an AD[6], bzw. PTA5

DEFINE PULLUPA IVAR[140]     'Definitionen fuer Pull-Up-Register und
DEFINE Carry IVAR[73]        'das Carry-Flag

DIM a,b,s,t BYTE             'temporaere Bytes


' Wenn die Analog-Digital-Ports AD[1] bis AD[6] verwendet werden, fuehren
' die internen Pull-Up-Widerstaende an den Digitalports oft zur Verfaelschung
' des Messwerts, da sie vom Kontroller vor jeder Messung deaktiviert und nach
' der Messung wieder aktiviert werden. Deshalb sollten die Widerstaende an
' den entsprechenden Ports mit Hilfe von PULLUPA abgeschaltet werden.

' Es ist auch zu beachten, dass ein Voltmeter an den Ports natuerlich ein
' anderes Messergebnis liefert als der ADU des Kontrollers, solange die
' Pull-Up-Widerstaende nicht deaktiviert wurden. Denn das Voltmeter misst
' die Spannung bei zugeschalteten Pull-Up-Widerstaenden. Die kurze AD-Mess-
' zeit spielt dabei meist kaum eine Rolle, da das Voltmeter zu traege ist.

' Mit dem folgenden Kommando wird der interne Pull-Up-Widerstand an PTA5
' (PORT[6]) ausgeschaltet, die Widerstaende an den anderen fuenf Digital-
' ports von PORTA (BYTEPORT[1]) bleiben dagegen weiterhin aktiv.

'      PTA..543210
PULLUPA=&b00011111


#endlos
  a=adpoti
  PRINT "AD[6]=";a;" => ";
  
  ' Um ein moeglichst exaktes Ergebnis zu liefern, muss zur Berechnung
  ' der Spannung die folgende Formel benutzt werden: (a+0.5)/256*5
  ' Der Anteil "0.5" gleicht den Aufloesungsverlust aus, der durch das
  ' Abschneiden von Nachkommastellen entsteht. Auf diese Weise wird der
  ' Mittelwert einer ADU-Spannungsstufe berechnet. Die "5" steht fuer
  ' die Versorgungsspannung des Kontrollers, die zugleich die Referenz-
  ' spannung des AD-Wandlers ist. Mit einem entsprechend angepassten
  ' Betriebssystem laesst sich der Kontroller auch bei 3 Volt betreiben.
  
  ' Durch Umstellen der Formel erhaelt man einen effektiveren Rechenweg:
  ' (a*5+2.5)/256
  
  ' Die folgende Subroutine multipliziert die Bytevariable "a" mit 5 und
  ' liefert das 16-Bit-Ergebnis in den Bytevariablen "a" und "b" zurueck.
  ' Dabei ist "a" das High-Byte und "b" das Low-Byte.
  
  GOSUB multiplizieren
   
  ' Jetzt muss der Anteil "2.5" addiert werden. Da das Ergebnis der Multi-
  ' plikation als Integerwert vorliegt, also keinen Nachkomma-Anteil besitzt,
  ' wird einfach 3 addiert. Der dadurch entstehende Fehler ist minimal.
  ' Ein eventueller Uebertrag muss zum High-Byte "a" hinzuaddiert werden.
  
  b=b+3
  SUB a,Carry      'Carry ist -1 oder 0, deshalb SUB statt ADD
  
  ' Nun muesste durch 256 geteilt und anschliessend das Ergebnis auf der
  ' seriellen Schnittstelle ausgegeben werden. Die Routine "printfixpoint"
  ' aus dem 16BITFIX-ZIP-Archiv kann 16-Bit-Festkommazahlen als dezimale
  ' Kommazahl auf der seriellen Schnittstelle ausgeben. Das 16-Bit-Fest-
  ' kommazahlen-Format besitzt einen 8 Bit grossen Ganzzahlanteil und
  ' einen ebenso grossen Nachkommaanteil. Als 16-Bit-Integerzahl betrachtet,
  ' wurde die Fliesskommazahl also mit 256 multipliziert. Aus diesem Grund
  ' koennen wir unser 16-Bit-Ergebnis direkt an die Routine uebergeben,
  ' ohne zunaechst durch 256 zu teilen und anschliessend die Division
  ' durch die Multiplikation mit 256 rueckgaengig zu machen.
  
  GOSUB printfixpoint
  
  ' Das Ergebnis ist recht genau. Der Fehler nach AD-Wandlung duerfte bei
  ' rund 0,2% vom Skalenendwert (5 Volt) liegen. Der AD-Umsetzer weist laut
  ' Datenblatt allerdings einen Fehler von bis zu +/- 1.5 LSB auf. Durch
  ' Mittelung mehrerer Messwerte laesst sich das Ergebnis verbessern.
  
  PRINT " Volt"
  PAUSE 25
GOTO endlos




' Die folgenden Routinen wurden aus 16BITFIX.ZIP uebernommen, wobei die
' Routine "multiplizieren" angepasst wurde.


' AD-Wert mit 5 multiplizieren

#multiplizieren
  mul8816 a,5
  b=pop
  removetos
  a=pop          'a.b=a*5
return

' zwei 8-Bit-Zahlen multiplizieren und 16-Bit-Ergebnis zurueckliefern

TABLE mul8816
  &h1206
  &had0c &h87ad &h0988 &h4287 &h9fad &h0186 &h5f83 &hae01
  &h83ff
TABEND

' Die 16-Bit-Festkommazahl a.b auf der seriellen Schnittstelle ausgeben

#printfixpoint
  print a;
  if b then
    put "."
    t=b
    while t    
      s=t*10                 'liefert nur das Lowbyte zurueck
      put "0"+(t-(s/10))/25  'also trickreich den Uebertrag ermitteln
      t=s
    wend
  end if
return

